import ldap as pyldap
import ldap.sasl as sasl
import ldap.modlist
import re
from datetime import datetime, date
from copy import deepcopy


class LDAP:

    def __init__(self, user, password,
                 host='ldaps://ldap.csh.rit.edu:636',
                 base='ou=Users,dc=csh,dc=rit,dc=edu',
                 bind='ou=Apps,dc=csh,dc=rit,dc=edu',
                 app=False,
                 objects=False):
        self.host = host
        self.base = base
        self.users = 'ou=Users,dc=csh,dc=rit,dc=edu'
        self.groups = 'ou=Groups,dc=csh,dc=rit,dc=edu'
        self.committees = 'ou=Committees,dc=csh,dc=rit,dc=edu'
        self.ldap = pyldap.initialize(host)
        self.ldap.set_option(pyldap.OPT_X_TLS_DEMAND, True)
        self.ldap.set_option(pyldap.OPT_DEBUG_LEVEL, 255)

        if app:
            self.ldap.simple_bind('uid=' + user + ',' + base, password)
            # self.ldap.simple_bind('uid='+user+','+bind, password)
        else:
            try:
                auth = sasl.gssapi("")

                self.ldap.sasl_interactive_bind_s("", auth)
                self.ldap.set_option(pyldap.OPT_DEBUG_LEVEL, 0)
            except pyldap.LDAPError, e:
                print 'Are you sure you\'ve run kinit?'
                print e

    def members(self, uid="*"):
        """ members() issues an ldap query for all users, and returns a dict
            for each matching entry. This can be quite slow, and takes roughly
            3s to complete. You may optionally restrict the scope by specifying
            a uid, which is roughly equivalent to a search(uid='foo')
        """
        entries = self.search(uid='*')
        if self.objects:
            return self.memberObjects(entries)
        result = []
        for entry in entries:
            result.append(entry[1])
        return result

    def member(self, user):
        """ Returns a user as a dict of attributes
        """
        try:
            member = self.search(uid=user)[0]
        except IndexError:
            return None
        if self.objects:
            return member
        return member[1]

    def eboard(self):
        """ Returns a list of eboard members formatted as a search
            inserts an extra ['commmittee'] attribute
        """
        # self.committee used as base because that's where eboard
        # info is kept
        committees = self.search(base=self.committees, cn='*')
        directors = []
        for committee in committees:
            for head in committee[1]['head']:
                director = self.search(dn=head)[0]
                director[1]['committee'] = committee[1]['cn'][0]
                directors.append(director)
        if self.objects:
            return self.memberObjects(directors)
        return directors

    def group(self, group_cn):
        members = self.search(base=self.groups, cn=group_cn)
        if len(members) == 0:
            return members
        else:
            member_dns = members[0][1]['member']
        members = []
        for member_dn in member_dns:
            members.append(self.search(dn=member_dn)[0])
        if self.objects:
            return self.memberObjects(members)
        return members

    def getGroups(self, member_dn):
        searchResult = self.search(base=self.groups, member=member_dn)
        if len(searchResult) == 0:
            return []

        groupList = []
        for group in searchResult:
            groupList.append(group[1]['cn'][0])
        return groupList

    def drinkAdmins(self):
        """ Returns a list of drink admins uids
        """
        admins = self.group('drink')
        return admins

    def rtps(self):
        rtps = self.group('rtp')
        return rtps

    def trimResult(self, result):
        return [x[1] for x in result]

    def search(self, base=False, trim=False, **kwargs):
        """ Returns matching entries for search in ldap
            structured as [(dn, {attributes})]
            UNLESS searching by dn, in which case the first match
            is returned
        """
        scope = pyldap.SCOPE_SUBTREE
        if not base:
            base = self.users

        filterstr = ''
        for key, value in kwargs.iteritems():
            filterstr += '({0}={1})'.format(key, value)
            if key == 'dn':
                filterstr = '(objectClass=*)'
                base = value
                scope = pyldap.SCOPE_BASE
                break

        if len(kwargs) > 1:
            filterstr = '(&'+filterstr+')'

        result = self.ldap.search_s(base,
                                    pyldap.SCOPE_SUBTREE,
                                    filterstr,
                                    ['*', '+'])
        if base == self.users:
            for member in result:
                groups = self.getGroups(member[0])
                member[1]['groups'] = groups
                if 'eboard' in member[1]['groups']:
                    eboard_search = self.search(base=self.committees,
                                                head=member[0])
                    member[1]['committee'] = eboard_search[0][1]['cn'][0]
        if self.objects:
            return self.memberObjects(result)
        finalResult = self.trimResult(result) if trim else result
        return finalResult

    def modify(self, uid, base=False, **kwargs):
        if not base:
            base = self.users
        dn = 'uid='+uid+',ou=Users,dc=csh,dc=rit,dc=edu'
        old_attrs = self.member(uid)
        new_attrs = deepcopy(old_attrs)

        for field, value in kwargs.iteritems():
            if field in old_attrs:
                new_attrs[field] = [str(value)]
        modlist = pyldap.modlist.modifyModlist(old_attrs, new_attrs)

        self.ldap.modify_s(dn, modlist)

    def memberObjects(self, searchResults):
        results = []
        for result in searchResults:
            newMember = Member(result, ldap=self)
            results.append(newMember)
        return results


def dateFromLDAPTimestamp(timestamp):
    """ Takes an LDAP date (In the form YYYYmmdd
        with whatever is after that) and returns a
        datetime.date object.
    """
    # only check the first 8 characters: YYYYmmdd
    numberOfCharacters = len("YYYYmmdd")
    timestamp = timestamp[:numberOfCharacters]
    try:
        day = datetime.strptime(timestamp, '%Y%m%d')
        return date(year=day.year, month=day.month, day=day.day)
    except:
        print(timestamp)



import requests
import json

class Webnews:
	def __init__(self, api_key, api_agent='A script using webnews.py'):
		self.payload = {'api_key'   : api_key
		          , 'api_agent' : api_agent
		          }
		self.headers = {'Accept' : 'application/json'}
		self.prefix = 'https://webnews.csh.rit.edu/'
	
	def _get(self, target, params={}):
		url = self.prefix + target
		payload = dict(self.payload.items() + params.items())
		return requests.get(url, params = payload, headers=self.headers, verify=False)

	def _post(self, target, params={}):
		url = self.prefix + target
		payload = dict(self.payload.items() + params.items())
		return requests.post(url, params = payload, headers=self.headers, verify=False)

	def _put(self, target, params={}):
		url = self.prefix + target
		payload = dict(self.payload.items() + params.items())
		return requests.put(url, params = payload, headers=self.headers, verify=False)

	def _delete(self, target, params={}):
		url = self.prefix + target
		payload = dict(self.payload.items() + params.items())
		return requests.delete(url, params = payload, headers=self.headers, verify=False)
	
	def newsgroups(self):
		url = self.prefix + 'newsgroups'
		r = requests.get(url,params = payload, headers=self.headers, verify=False)
		deserialized = json.loads(r.text)
		return self._get('newsgroups')

	def newsgroup_names(self):
		names = []
		groupDicts = self.newsgroups()[u'newsgroups']
		for groupDict in groupDicts:
			name = groupDict[u'name']
			names.append(name)
		return names

	def search(self, more=True, **kwargs):
		while more:
			r = json.loads(self._get('search', params=kwargs).content)
			posts = r['posts_older']
			more = r['more_older']
			if more:
				kwargs['from_older'] = posts[-1]['post']['date']
			yield posts

	def user(self):
		return json.loads(self._get('user').content)['user']

	def newspost(self, newsgroup, num):
		r = self._get(newsgroup+'/'+str(num), {'html_body':True})
		return json.loads(r.content)['post']
	def get_unread_counts(self):
		return json.loads(self._get('unread_counts').content)['unread_counts']

	def get_newsgroup_index(self, newsgroup):
		return json.loads(self._get(newsgroup+'/index', {'newsgroup':newsgroup}).content)

	def get_last_date(self, posts):
		return posts[-1][u'post'][u'date']

	def compose(self, newsgroup, subject, **kwargs):
		kwargs['newsgroup'] = newsgroup
		kwargs['subject'] = subject
		self._post('compose', params = kwargs)

	def mark_read(self, **kwargs):
		self._put('mark_read', kwargs)

	def stick(self, newsgroup, number, time):
		self._put(newsgroup+'/'+number+'/sticky', {'sticky_until':time})

	def unstick(self, newsgroup, number):
		self._put(newsgroup+'/'+number+'/sticky', {'unstick':True})
	
	def star(self, newsgroup, number):
		self._put(newsgroup+'/'+number+'/star')
	
	def delete_post(self, newsgroup, number, reason='I do what I want'):
		params = { 'reason' : reason 
				 , 'confirm_cancel' : True
				 , 'posting_host' : 'wannamaker'
				 }
		self._delete(newsgroup+'/'+str(number), params)

	def activity(self):
		a = self._get('activity')
		return json.loads(a.content)['activity']




from ldapapi import LDAP as ldap

class Member(object):
    def __init__(self, member, ldap=None):
        """ Creates and returns a member object from which LDAP fields
            are accessible as properties. If you supply an LDAP connection,
            the object will use that connection to reload its data and
            modify its fields if you choose.
        """
        self.specialFields = ("memberDict", "ldap")
        if len(member) < 2:
            self.memberDict = {}
        else:
            self.memberDict = member[1]
        self.ldap = ldap

    def __getattr__(self, attribute):
        """ Accesses the internal dictionary representation of
            a member and returns whatever data type it represents.
        """
        if (attribute == "specialFields" or attribute in self.specialFields):
            return object.__getattribute__(self, attribute)
        try:
            # Grab the object at that key. It will be a list,
            # if it exists.
            attributes = self.memberDict[attribute]

            # If we do get a list, and it only
            # contains one thing, just return that
            # one thing.
            if len(attributes) == 1:
                attribute = attributes[0]
                if attribute.isdigit():
                    attribute = int(attribute)
                return attribute
            return attributes
        # If there was an error (i.e. that member doesn't have that
        # key in their LDAP store), then return None. We couldn't get it.
        except (KeyError, IndexError):
            return None

    def __setattr__(self, attribute, value):
        """ When setting an attribute with 'member.field = "value"',
            access the internal ldap connection from the constructor
            and modify that parameter.
        """
        if (attribute == "specialFields" or attribute in self.specialFields):
            return object.__setattr__(self, attribute, value)
        if attribute in ("memberDict", "ldap"):
            object.__setattr__(self, attribute, value)
            return
        if not self.ldap:
            return
        kwargs = {attribute: value}
        self.ldap.modify(uid=self.uid, **kwargs)
        self.memberDict[attribute] = value

    def fields(self):
        """ Returns all of the keys in the internal dictionary.
        """
        return self.memberDict.keys()

    def isActive(self):
        """ Is the user active?
        """
        return bool(self.active)

    def isAlumni(self):
        """ Is the user an alumnus/a?
        """
        return bool(self.alumni)

    def isDrinkAdmin(self):
        """ Is the user a drink admin?
        """
        return bool(self.drinkAdmin)

    def isOnFloor(self):
        """ Is the user on floor?
        """
        return bool(self.onfloor)

    def isEboard(self):
        """ Is the user on Eboard?
        """
        return 'eboard' in self.groups

    def isRTP(self):
        """ Is the user an RTP?
        """
        return 'rtp' in self.groups

    def isBirthday(self):
        """ Is it the user's birthday today?
        """
        if not self.birthday:
            return False
        birthday = self.birthdate()
        today = date.today()
        return (birthday.month == today.month and
                birthday.day == today.day)

    def birthdate(self):
        """ Converts the user's birthday (if it exists) to a datetime.date
            object that can easily be compared with other dates.
        """
        if not self.birthday:
            return None
        return ldap.dateFromLDAPTimestamp(self.birthday)

    def joindate(self):
        """ Converts the user's join date (if it exists) to a datetime.date
            object that can easily be compared with other dates.
        """
        if not self.memberSince:
            return None
        joined = self.memberSince
        return ldap.dateFromLDAPTimestamp(joined)

    def age(self):
        """ Returns the user's age, determined by their birthdate()
        """
        if not self.birthdate():
            return -1
        adjuster = 0
        today = date.today()
        birthday = self.birthdate()
        if today.month == birthday.month:
            if today.day < birthday.day:
                adjuster -= 1
        elif today.month < birthday.month:
            adjuster -= 1
        return (today.year - birthday.year) + adjuster

    def reload(self):
        """ If there is an LDAP connection, query it for another
            instance of this member and set its internal dictionary
            to that result.
        """
        if not self.ldap:
            return
        self.memberDict = self.ldap.member(self.uid)

    def fullName(self):
        """ Returns a reliable full name (firstName lastName) for every
            member (as of the writing of this comment.)
        """
        if self.givenName and self.sn:
            return "{0} {1}".format(self.givenName, self.sn)
        if self.givenName:
            return self.givenName
        if self.sn:
            return self.sn
        return self.uid

    def __str__(self):
        """ Constructs a string representation of this person, containing
            every key and value in their internal dictionary.
        """
        string = ""
        for key in self.memberDict.keys():
            value = self.__getattr__(key)
            string += str(key) + ": " + str(value) + "\n"
        return string
